//-------------------------------------------------------------------------------------------------
//	Copyright  2008 Jeromy Walsh
//
//  You are hereby authorized to modify and/or redistribute the code contained within this file
//	so long as this header stays intact.  Any bugfixes or modifications you make to this code
//	can be sent to  "dragonwalsh .at. yahoo .dot. com"
//-------------------------------------------------------------------------------------------------

#pragma once

enum NormalMode
{
	NormalMode_FilterNormals,
	NormalMode_CpuNormals,
	NormalMode_GpuNormals,
	NormalMode_StreamNormals
};


//-------------------------------------------------------------------------------------------------
//	[Structures]
//-------------------------------------------------------------------------------------------------

struct MeshVertex	// 28 Bytes per Vertex
{
	D3DXVECTOR3 pos;
	D3DXVECTOR3 norm;
	unsigned  i;
};

struct FilterVertex	// 8 Bytes per Vertex
{
	float x, z;  
};

//-------------------------------------------------------------------------------------------------
//	[Classes]
//-------------------------------------------------------------------------------------------------

class Heightfield
{
public:
	
	 Heightfield( void );
	~Heightfield( void );

	void	Init	( float m_MetersPerVertex, ID3D10Device* pDevice, ID3D10Effect* pEffect );
	void	Update	( float elapsedTime );
	void	Draw	();
	void	Release	();
	void	Clear	();

	// What mode are we using to calculate normals
	NormalMode Mode;
	bool IsWireframe;

protected:

	HRESULT hr;	// Used for catching return values

	// Vertex/Index Buffer Creation
	void	CreateIndexBuffer		( int numVertsWide, int numRows);
	void	CreateVertexBuffers		( int numVertices );
	void	CreateShaderResources	( int numSurfaces );
	void 	ComputeNormals			();
	void	ComputeSurfaceNormals	();
	void 	ComputeGPUNormals		();
	void 	ComputeCPUNormals		();
	void	GenerateMeshData		();
	void	FillMeshVB				();
	void	FillFilterBuffer		();
	void	InitHeightfieldData		(D3DXVECTOR2* pData);
	
	// Information about the heightfield
	int		m_NumVertsDeep;
	int		m_NumVertsWide;
	int		m_Width;			// The width of the terrain in meters
	int		m_Depth;			// The depth of the terrain in meters
	float	m_MetersPerVertex;	// The distance between vertices
	
	float*			m_pHeights;
	int*			m_pIndices;
	MeshVertex*		m_pVertices;
	D3DXVECTOR3*	m_pSurfaceNormals;

	// DirectX 10 Generic Resources
	ID3D10Device*				m_pDevice;			// DirectX Device
	ID3D10Effect*				m_pEffect;			// Effect
	ID3D10Buffer*				m_pIndexBuffer;		// Index Buffer for Mesh and Heightfield
	ID3D10EffectScalarVariable*	m_pMetersPerVertex;
	ID3D10EffectScalarVariable*	m_pNumVertsWide;
	ID3D10EffectScalarVariable*	m_pNumVertsDeep;

	// Optional Fields
	ID3D10RasterizerState*		m_pRasterizerState;
	ID3D10RasterizerState*		m_pWireframeState;
	ID3D10EffectVectorVariable*	m_pMaterialDiffuseColor;

	// Heightfield Resources
	ID3D10Buffer*				m_pHeightfieldVB;		// Vertex Buffer for heightfield
	ID3D10InputLayout*			m_pHeightfieldIL;		// Input Layout for Mesh IA
	ID3D10EffectTechnique*		m_pFilterSimpleTech;	// The Filter Technique
	ID3D10Buffer*				m_pHeightBuffer;		// Buffer for height data
	ID3D10ShaderResourceView*	m_pHeightBufferRV;		// View of Buffer
	ID3D10EffectShaderResourceVariable* m_pHeightsRV;	// Effect Binding

	// Mesh Resources
	ID3D10Buffer*				m_pMeshVB;				// Vertex Buffer for mesh
	ID3D10InputLayout*			m_pMeshIL;				// Input Layout for Mesh IA
	ID3D10EffectTechnique*		m_pMeshSimpleTech;
	ID3D10EffectTechnique*		m_pMeshWithNormalMapTech;
	ID3D10EffectTechnique*		m_pMeshWithNormalMapSOTech;
	ID3D10Buffer*				m_pNormalBuffer;
	ID3D10ShaderResourceView*	m_pNormalBufferRV;
	ID3D10Buffer*				m_pNormalBufferSO;
	ID3D10ShaderResourceView*	m_pNormalBufferRVSO;
	ID3D10EffectShaderResourceVariable* m_pSurfaceNormalsRV;
};




